import subprocess
import re
import os
import glob
from datetime import datetime

def get_user_from_file(interface):
    """从配置文件中提取宽带账号"""
    file_path = f"/etc/sysconfig/network-scripts/ifcfg-{interface}"
    try:
        with open(file_path, 'r') as file:
            for line in file:
                if line.startswith("USER="):
                    return line.split('=')[1].strip()
    except FileNotFoundError:
        print(f"File {file_path} not found.")
    return "N/A"

def get_pid(interface):
    """通过ps命令查询PID号"""
    try:
        result = subprocess.check_output(['ps', '-ef']).decode('utf-8')
        for line in result.split('\n'):
            if f"pppd call {interface}" in line and "grep" not in line:
                return int(line.split()[1])
    except subprocess.CalledProcessError as e:
        print(f"Error fetching PID for {interface}: {e}")
    return "N/A"

def get_ip_address(interface):
    """通过ip命令查询IP地址"""
    try:
        # 使用 ip addr 命令获取指定 interface 对应的 ppp 接口信息
        result = subprocess.check_output(['ip', 'addr', 'show', interface]).decode('utf-8')
        # 使用正则表达式提取 IP 地址信息
        ip_match = re.search(r'inet\s+(\d+\.\d+\.\d+\.\d+)', result)
        if ip_match:
            ip_address = ip_match.group(1)
            return ip_address
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching PPP interface info for interface {interface}: {e}")
        return "N/A"

def get_dial_status(pid, log_file):
    """通过PID号获取拨号反馈信息"""
    status = "N/A"
    try:
        # 使用 grep 命令过滤包含特定进程ID的行
        result = subprocess.check_output(['cat', log_file]).decode('utf-8')
        lines = result.split('\n')
        for line in lines:
            if f"pppd[{pid}]" in line and "authentication" in line:
                # 使用正则表达式提取不同的 authentication 状态及其后面的消息作为状态
                succeeded_match = re.search(r'authentication\s*(.+)', line)
                pending_match = re.search(r'authentication\s+succeeded:\s*(.+)', line)
                failed_match = re.search(r'authentication\s+failed:\s*(.+)', line)
                timeout_match = re.search(r'authentication\s+timeout:\s*(.+)', line)
                
                if succeeded_match:
                    status = "" + succeeded_match.group(1)
                    break
                elif failed_match:
                    status = "failed: " + failed_match.group(1)
                    break
                elif pending_match:
                    status = "pending: " + pending_match.group(1)
                    break
                elif timeout_match:
                    status = "timeout: " + timeout_match.group(1)
                    break
    except subprocess.CalledProcessError as e:
        print(f"Error fetching dial status: {e}")
    return status

def get_interfaces():
    """获取接口编号"""
    interface_files = glob.glob("/etc/sysconfig/network-scripts/ifcfg-ppp*")
    interfaces = [os.path.basename(f).replace("ifcfg-", "") for f in interface_files]
    return interfaces

def main():
    interfaces = get_interfaces()
    results = []
    # 获取当前日期并格式化为字符串
    current_date = datetime.now().strftime("%Y%m%d")
    log_file = f"/var/log/pppoe-{current_date}.log"  # 构建日志文件路径

    for interface in interfaces:
        user = get_user_from_file(interface)
        pid = get_pid(interface)
        ip_address = get_ip_address(interface)
        dial_status = get_dial_status(pid, log_file)
        
        results.append({
            "interface": interface,
            "user": user,
            "pid": pid,
            "ip_address": ip_address,
            "dial_status": dial_status
        })
    
    # 打印结果
    print(f"{'接口':<10} {'USER':<20} {'PID':<10} {'IP地址':<20} {'日志':<20}")
    for result in results:
        print(f"{result['interface']:<10} {result['user']:<20} {result['pid']:<10} {result['ip_address']:<20} {result['dial_status']:<20}")

if __name__ == "__main__":
    main()
